#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

static char filename[256];

void open_file(GtkWidget * parentwindow, char * file_name)
{
	int fd, len;
	char * buf[1024];
	GtkTextView * textview1 = lookup_widget(parentwindow, "textview1");
	GtkTextBuffer * textbuffer = gtk_text_view_get_buffer(textview1);
	if ((fd = open(file_name, O_RDONLY, 0)) < 0)
	{
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(parentwindow, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
			"Невозможно открыть файл %s", file_name);
		gtk_dialog_run (GTK_DIALOG (dialog));		
		gtk_widget_destroy (dialog);
		return;
	}	
	strncpy(filename, file_name, 256);
	gtk_text_buffer_set_text(textbuffer, "", 0);
	while((len = read(fd, buf, 1024)) != 0)
		gtk_text_buffer_insert_at_cursor(textbuffer, buf, len);	
	close(fd);	
}

void save_file(GtkWidget * parentwindow, char * file_name)
{
	int fd, len;
	GtkTextIter start, end;
	gchar * buf;
	GtkTextView * textview1 = lookup_widget(parentwindow, "textview1");
	GtkTextBuffer * textbuffer = gtk_text_view_get_buffer(textview1);
	if ((fd = open(file_name, O_WRONLY|O_CREAT|O_TRUNC, 0666)) < 0)
	{
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(parentwindow, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
			"Невозможно открыть файл %s", file_name);
		gtk_dialog_run (GTK_DIALOG (dialog));		
		gtk_widget_destroy (dialog);
		return;
	}
	gtk_text_buffer_get_start_iter(textbuffer, &start);
	gtk_text_buffer_get_end_iter(textbuffer, &end);
	buf = gtk_text_buffer_get_text(textbuffer, &start, &end, FALSE);
	write(fd, buf, strlen(buf));
	close(fd);	
	g_free(buf);
}

void
on_new1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkTextView * textview1 = lookup_widget(menuitem, "textview1");
	GtkTextBuffer * textbuffer = gtk_text_view_get_buffer(textview1);
	gtk_text_buffer_set_text(textbuffer, "", 0);
	filename[0] = 0;
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * dialog;
	GtkWidget * window = lookup_widget(menuitem, "window1");
	dialog = gtk_file_chooser_dialog_new("Открыть файл", window,
		GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
	{
		char * file_name;
	    file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    	open_file(window, file_name);
		g_free (file_name);
	}
	gtk_widget_destroy(dialog);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * window = lookup_widget(menuitem, "window1");
	if(filename[0] != 0)
		save_file(window, filename);
	else
		on_save_as1_activate(menuitem, user_data);
}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * dialog;
	GtkWidget * window = lookup_widget(menuitem, "window1");
	dialog = gtk_file_chooser_dialog_new ("Сохранить как...", window,
		GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,  NULL);
	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
	{
		char * file_name;
	    file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    	save_file(window, file_name);
		g_free (file_name);
		strncpy(filename, file_name, 256);
	}
	gtk_widget_destroy(dialog);
}

void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_window1_destroy(menuitem, user_data);
}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkTextView * textview1 = lookup_widget(menuitem, "textview1");
	GtkTextBuffer * textbuffer = gtk_text_view_get_buffer(textview1);
	GtkClipboard * cb = gtk_clipboard_get(gdk_atom_name("CLIPBOARD"));
	gtk_text_buffer_cut_clipboard(textbuffer, cb, TRUE);
}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkTextView * textview1 = lookup_widget(menuitem, "textview1");
	GtkTextBuffer * textbuffer = gtk_text_view_get_buffer(textview1);
	GtkClipboard * cb = gtk_clipboard_get(gdk_atom_name("CLIPBOARD"));
	gtk_text_buffer_copy_clipboard(textbuffer, cb);
}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkTextView * textview1 = lookup_widget(menuitem, "textview1");
	GtkTextBuffer * textbuffer = gtk_text_view_get_buffer(textview1);
	GtkClipboard * cb = gtk_clipboard_get(gdk_atom_name("CLIPBOARD"));
	gtk_text_buffer_paste_clipboard(textbuffer, cb, NULL, TRUE);
}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}

void
on_fontselect_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkFontSelectionDialog * dialog = gtk_font_selection_dialog_new("Выбор шрифта");
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
	{
		GtkTextView * textview1 = lookup_widget(menuitem, "textview1");
		char * font_name = gtk_font_selection_dialog_get_font_name(dialog);
		PangoFontDescription * font_desc = pango_font_description_from_string(font_name);	
		gtk_widget_modify_font (textview1, font_desc);
  		pango_font_description_free (font_desc);
		g_free(font_name);
	}
	gtk_widget_destroy(dialog);
}
